import React, { useState, useEffect } from 'react';
import {ComponentDefinition, FormComponent} from '@formbird/types';
import { convertToCustomElement } from '../utils/CustomElementWrapper';
import { XGrid } from '@material-ui/x-grid';
import { ChangedDocumentService, SearchService } from '@formbird/services';

interface FormbirdGridColumn { 
  displayName: string;
  field: string;
  type?: string;
  width?: number;
  sort?: {
    direction?: string;
    precedence?: number;
  };
  urlOpenIn?: string;
  href?: string;
}

interface DatatablesComponentDefinition extends ComponentDefinition {
  dataSource?: string;
  handlebarsHelpers?: any;
  searchOptions?: any;
  responsive?: boolean;
  showHeadings?: boolean;
  forcePageInfo?: any;
  showSelected?: boolean;
  showSearch?: boolean;
  noChildDocLink?: boolean;
  showReload?: boolean;
  pageSelect?: any;
  allSelect?: any;
  enableColumnResizing?: boolean;
  enableColumnMoving?: boolean;
  multiSelect?: boolean;
  columnSearch?: boolean;
  delayBeforeCallNum?: number;
  rowsPerPage?: number;
  lengthMenu?: number[];
  selectOnLoad?: boolean;
  disableAutoDisplay?: boolean;
  refreshInterval?: number;
  backgroundRefresh?: boolean;
  enforceWidth?: boolean;
  hideNoData?: boolean;
  templateTarget?: string;
  template?: [any?];
  childLabel?: string;
  gridColumns?: [FormbirdGridColumn?];
  filter?: string;
  linkFunction?: any;
  updateWatchFields?: any;
  exportTypes?: any;
  headerRotate?: any;
  headerHeight?: any;
  fontSize?: any;
  exportSettings?: any;
  queryName?: string;
  headerWrapping?: boolean;
}

const ReXGridComponent = ({
  document,
  fieldValue,
  template,
  fieldName,
  formParameters,
  key,
  responsiveLayouts,
  message,
  componentDefinition
}) => {

  const [value, setValue] = useState();
  const [columns, setColumns] = useState([]);
  const [rows, setRows] = useState([]);
  const [ids, setIds] = useState([]);
  const [loading, setLoading] = useState(false);
  const [rowsPerPageOptions, setRowsPerPageOptions] = useState([5, 10, 15]);


  
  // [
  //   { field: 'id', headerName: 'ID', width: 90 },
  //   {
  //     field: 'firstName',
  //     headerName: 'First name',
  //     width: 150,
  //     editable: true,
  //   },
  //   {
  //     field: 'lastName',
  //     headerName: 'Last name',
  //     width: 150,
  //     editable: true,
  //   },
  //   {
  //     field: 'age',
  //     headerName: 'Age',
  //     type: 'number',
  //     width: 110,
  //     editable: true,
  //   },
  //   {
  //     field: 'fullName',
  //     headerName: 'Full name',
  //     description: 'This column has a value getter and is not sortable.',
  //     sortable: false,
  //     width: 160,
  //     valueGetter: (params) =>
  //       `${params.getValue(params.id, 'firstName') || ''} ${
  //         params.getValue(params.id, 'lastName') || ''
  //       }`,
  //   },
  // ];

  useEffect(() => {
    if (!loading) {
      const gridColumns = [];

      if ( !(typeof componentDefinition.gridColumns === 'undefined' || componentDefinition.gridColumns === null)) {
        for (let i = 0; i < componentDefinition.gridColumns.length; i++) {
          const col = componentDefinition.gridColumns[i];
          let width = 150;
          if (col.width) { width = col.width; }

          let valueFormatter;

          const dotSeparator = col.field.indexOf('.');

          let field;

          if (dotSeparator > 0) {
            field = col.field.substring(0, dotSeparator);
            valueFormatter = (params) => {
              return params.colDef.type === "date" ? new Date(params.row?.systemHeader.createdDate) : params.row?.systemHeader.summaryName;
            }
          } else {
            field = col.field;
          }

          const column = {
            field: col.field,
            headerName: col.displayName,
            flex: 1,
            type: col.type,
            valueFormatter: valueFormatter
          };
          gridColumns.push(column);
        }

        setColumns(gridColumns);
      }
      const anyWindow: any = window;
      const searchService: SearchService = (anyWindow.FormbirdServiceInjector).get('SearchService');

      // columns = componentDefinition.gridColumns.map(columDef => {
      //   return {
      //     field: columnDef.field,
      //     headerName: columnDef.displayName,
      //     type: columnDef.type
      //   }
      // });

      const options = {
        filter: componentDefinition.filter,
        includeDeleted: false,
        searchOfflineOnly: false,
      };
      setLoading(true);
      searchService.search(options).then((results) => {
        console.log('results = ', results);
        const data = [];
        if (results.data && results.data.hits && results.data.hits.hits) {
          results.data.hits.hits.forEach((item) => {
            const source = item._source;
            if (ids.indexOf(source.documentId) < 0) {
              source.id = source.documentId;
              data.push(source);
              setIds(source.documentId);
            }
          });
        }
        setRows(data);
      }, (err) => {
        setLoading(false);
        console.error('error fetching data', err);
      });
    }
  }, []);

  return (
    <div style={{ height: '800px', width: '100%' }}>
    
    <XGrid
      rows={rows}
      columns={columns}
      loading={rows.length === 0}
      pageSize={componentDefinition.rowsPerPage}
      rowsPerPageOptions={[5, 10, 20]}
      pagination
      // onChange={(e: any) => {
      //   const newValue = e.target.value;
      //   const anyWindow: any = window;
      //   const changedDocumentService: ChangedDocumentService = (anyWindow.FormbirdServiceInjector).get('ChangedDocumentService');

      //   changedDocumentService.valueChanged({
      //     document,
      //     fieldName,
      //     fieldValue,
      //     formParameters,
      //     template,
      //     componentDefinition,
      //     key
      //   }, newValue);

      //   // setValue(newValue);
      // }}
      // {...componentProps}
    />
    </div>
  );
};

// register the component as a custom element so it can be shown in formbird
convertToCustomElement('re-x-grid', ReXGridComponent);

export default ReXGridComponent;
